/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import javax.servlet.jsp.tagext.TagData;
import javax.servlet.jsp.tagext.TagExtraInfo;
import javax.servlet.jsp.tagext.VariableInfo;


/**
 * This is the tag extra info that describes the attribute to variable mapping that will occur within the
 * BreadCrumbTrailTag.  This tag extra info will define two variables (that can be used in the body of the
 * BreadCrumbTrailTag): <ol> <li>{@link BreadCrumbTrailTag#BREADCRUMB_FORMAT_NODE} variable that will contain a
 * reference to the {@link BreadCrumbNode} that should be formatted</li> <li>{@link
 * BreadCrumbTrailTag#BREADCRUMB_FORMAT_STYLE_CLASS} variable that will contain a reference to the String that
 * represents the style that will be used.  Please note that this string can be an empty string or <tt>null</tt> to
 * represent no styles to be applied</li> </ol>
 * <p/>
 * <b>Thread Safety: </b> this class is immutable and thread safe.
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class BreadCrumbTrailTagExtraInfo extends TagExtraInfo {
    /**
     * This constant contains the static return value of the {@link #getVariableInfo(javax.servlet.jsp.tagext.TagData)}
     * method.
     */
    private static final VariableInfo[] VARIABLE_INFO = new VariableInfo[]{
        new VariableInfo(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, BreadCrumbNode.class.getName(), true, VariableInfo.NESTED),
        new VariableInfo(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, String.class.getName(), true, VariableInfo.NESTED)
    };

    /**
     * This method will define the variables that can be used in the body of the {@link BreadCrumbTrailTag}.
     *
     * @param data A non-<tt>null</tt> tag data to use (not used by this implementation)
     *
     * @return Always a 2-element array representing the two elements to define
     */
    public VariableInfo[] getVariableInfo(final TagData data) {
        return VARIABLE_INFO;
    }
}
