/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import junit.framework.TestCase;


/**
 * This class contains all test cases for {@link BreadCrumbException}.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class BreadCrumbExceptionTest extends TestCase {
    /**
     * This is a value used as valid message argument in constructor testcases.
     */
    private static final String MESSAGE = "The message";
    /**
     * This is a value used as valid cause arguments in constructors.
     */
    private static final Exception CAUSE = new Exception("x");
    /**
     * This is an empty message string for checking that empty messages do not fail.
     */
    private static final String EMPTY_MESSAGE = "  ";

    /**
     * This method tests the correct behavior of {@link BreadCrumbException#BreadCrumbException(String)}.
     */
    public void testOneArgCtor() {
        BreadCrumbException ex = new BreadCrumbException(MESSAGE);
        assertEquals(MESSAGE, ex.getMessage());

        // check with empty message
        ex = new BreadCrumbException(EMPTY_MESSAGE);
        assertEquals(EMPTY_MESSAGE, ex.getMessage());

        // check with null message
        ex = new BreadCrumbException(null);
        assertEquals(null, ex.getMessage());

    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbException#BreadCrumbException(String,Throwable)}.
     */
    public void testTwoArgCtor() {
        BreadCrumbException ex = new BreadCrumbException(MESSAGE, CAUSE);
        assertEquals(MESSAGE, ex.getMessage());
        assertEquals(CAUSE, ex.getRootCause());

        // check with empty message
        ex = new BreadCrumbException(EMPTY_MESSAGE, CAUSE);
        assertEquals(EMPTY_MESSAGE, ex.getMessage());
        assertEquals(CAUSE, ex.getRootCause());
        // check with null message
        ex = new BreadCrumbException(null, CAUSE);
        assertEquals(null, ex.getMessage());
        assertEquals(CAUSE, ex.getRootCause());
        // check with null cause
        ex = new BreadCrumbException(EMPTY_MESSAGE, null);
        assertEquals(EMPTY_MESSAGE, ex.getMessage());
        assertEquals(null, ex.getRootCause());
    }
}
