/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodediscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BaseBreadCrumbTrailTagTest;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;

import javax.servlet.jsp.tagext.TagSupport;


/**
 * This class contains all test cases for {@link HttpServletNodeDiscoveryTag}.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class HttpServletNodeDiscoveryTagTest extends BaseBreadCrumbTrailTagTest {
    /**
     * This is a value used as valid title argument in testcases.
     */
    private static final String TITLE = "__title__";
    /**
     * This is an empty string for checking empty String argument behavior.
     */
    private static final String EMPTY_STRING = "  ";
    /**
     * This is the instance to be tested, created in {@link #setUp()}.
     */
    private HttpServletNodeDiscoveryTag nodeDiscovery;

    /**
     * This method tests the correct behavior of {@link HttpServletNodeDiscoveryTag#HttpServletNodeDiscoveryTag()}.
     */
    public void testCtor() {
        new HttpServletNodeDiscoveryTag();
    }

    /**
     * This method tests the correct behavior of {@link HttpServletNodeDiscoveryTag#getTitle()} and {@link
     * HttpServletNodeDiscoveryTag#setTitle(String)} .
     */
    public void testGetSetTitle() {
        assertEquals(null, nodeDiscovery.getTitle());
        nodeDiscovery.setTitle(TITLE);
        assertEquals(TITLE, nodeDiscovery.getTitle());
        nodeDiscovery.setTitle(null);
        assertEquals(null, nodeDiscovery.getTitle());
        nodeDiscovery.setTitle(EMPTY_STRING);
        assertEquals(null, nodeDiscovery.getTitle());
    }

    /**
     * This method tests the correct behavior of {@link HttpServletNodeDiscoveryTag#doStartTag()}.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTag() throws Exception {
        pageContext.removeAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE);
        assertEquals(TagSupport.SKIP_BODY, nodeDiscovery.doStartTag());

        BreadCrumbNode discoveredNode = (BreadCrumbNode) pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE);
        final String queryString = request.getQueryString();
        final String expected = request.getRequestURI() + (queryString != null ? "?" + queryString : "");

        assertNotNull(discoveredNode);
        assertEquals(null, discoveredNode.getTitle());
        assertEquals(expected, discoveredNode.getUrl());

        nodeDiscovery.setTitle(TITLE);
        assertEquals(TagSupport.SKIP_BODY, nodeDiscovery.doStartTag());

        discoveredNode = (BreadCrumbNode) pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE);

        assertNotNull(discoveredNode);
        assertEquals(TITLE, discoveredNode.getTitle());
        assertEquals(expected, discoveredNode.getUrl());
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        nodeDiscovery = new HttpServletNodeDiscoveryTag();
        nodeDiscovery.setPageContext(pageContext);
    }

}
