/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodediscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BaseBreadCrumbTrailTagTest;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;

import javax.servlet.jsp.tagext.BodyContent;
import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.tagext.TagSupport;


/**
 * This class contains all test cases for {@link InlineTextNodeDiscoveryTag}.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class InlineTextNodeDiscoveryTagTest extends BaseBreadCrumbTrailTagTest {
    /**
     * This is a value used as valid title argument in testcases.
     */
    private static final String TITLE = "__title__";
    /**
     * This is a value used as valid url argument in testcases.
     */
    private static final String URL = "u://rl";
    /**
     * This is an empty string for checking empty String argument behavior.
     */
    private static final String EMPTY_STRING = "  ";
    /**
     * This is the instance to be tested, created in {@link #setUp()}.
     */
    private InlineTextNodeDiscoveryTag nodeDiscovery;
    /**
     * This is the body inside the tag, initialized in {@link #setUp()}.
     */
    private BodyContent bodyContent;

    /**
     * This method tests the correct behavior of {@link InlineTextNodeDiscoveryTag#InlineTextNodeDiscoveryTag()}.
     */
    public void testCtor() {
        new InlineTextNodeDiscoveryTag();
    }

    /**
     * This method tests the correct behavior of {@link InlineTextNodeDiscoveryTag#getTitle()} and {@link
     * InlineTextNodeDiscoveryTag#setTitle(String)} .
     */
    public void testGetSetTitle() {
        assertEquals(null, nodeDiscovery.getTitle());
        nodeDiscovery.setTitle(InlineTextNodeDiscoveryTagTest.TITLE);
        assertEquals(InlineTextNodeDiscoveryTagTest.TITLE, nodeDiscovery.getTitle());
        nodeDiscovery.setTitle(null);
        assertEquals(null, nodeDiscovery.getTitle());
        nodeDiscovery.setTitle(InlineTextNodeDiscoveryTagTest.EMPTY_STRING);
        assertEquals(null, nodeDiscovery.getTitle());
    }

    /**
     * This method tests the correct behavior of {@link InlineTextNodeDiscoveryTag#doStartTag()}.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTag() throws Exception {
        assertEquals(BodyTagSupport.EVAL_BODY_BUFFERED, nodeDiscovery.doStartTag());
    }

    /**
     * This method tests the correct behavior of {@link InlineTextNodeDiscoveryTag#doAfterBody()}.
     * <p/>
     * <b>Special scenario:</b>the body evaluated to an empty string
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoAfterBodyBodyEmpty() throws Exception {
        try {
            nodeDiscovery.doAfterBody();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link InlineTextNodeDiscoveryTag#doAfterBody()}.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoAfterBody() throws Exception {

        // url in body
        bodyContent.print(URL);
        assertEquals(TagSupport.SKIP_BODY, nodeDiscovery.doAfterBody());

        BreadCrumbNode discoveredNode = (BreadCrumbNode) pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE);

        assertNotNull(discoveredNode);
        assertEquals(null, discoveredNode.getTitle());
        assertEquals(URL, discoveredNode.getUrl());

        //title set
        nodeDiscovery.setTitle(TITLE);
        assertEquals(TagSupport.SKIP_BODY, nodeDiscovery.doAfterBody());

        discoveredNode = (BreadCrumbNode) pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE);

        assertNotNull(discoveredNode);
        assertEquals(TITLE, discoveredNode.getTitle());
        assertEquals(URL, discoveredNode.getUrl());
    }

    /**
     * This method does the cleanup after test run.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void tearDown() throws Exception {
        super.tearDown();
        pageContext.popBody();
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        nodeDiscovery = new InlineTextNodeDiscoveryTag();
        nodeDiscovery.setPageContext(pageContext);
        bodyContent = pageContext.pushBody();
        pageContext.removeAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE);
        nodeDiscovery.setBodyContent(bodyContent);
    }
}
