/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import com.meterware.httpunit.WebResponse;
import com.topcoder.web.ui.tag.breadcrumbtrail.BaseBreadCrumbTrailTagTest;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;

import javax.servlet.jsp.tagext.TagSupport;


/**
 * This class contains all test cases for {@link HtmlAnchorNodeFormatterTag}.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class HtmlAnchorNodeFormatterTagTest extends BaseBreadCrumbTrailTagTest {
    /**
     * This is a value used as valid title argument in testcases.
     */
    private static final String TITLE = "__title__";
    /**
     * This is a value used as valid url argument in testcases.
     */
    private static final String URL = "u://rl";
    /**
     * This is a format string used in test case.
     */
    private static final String FORMAT = "--format--";
    /**
     * This is the instance to be tested, created in {@link #setUp()}.
     */
    private HtmlAnchorNodeFormatterTag nodeFormatterTag;
    /**
     * This is the current node to be rendered, initialized in {@link #setUp()}.
     */
    private BreadCrumbNode currentNode;

    /**
     * This method tests the correct behavior of {@link HtmlAnchorNodeFormatterTag#HtmlAnchorNodeFormatterTag()}.
     */
    public void testCtor() {
        new HtmlAnchorNodeFormatterTag();
    }

    /**
     * This method tests the correct behavior of {@link HtmlAnchorNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>no current node exists
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagNoNode() throws Exception {
        try {
            nodeFormatterTag.doStartTag();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link HtmlAnchorNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>current node context attribute has wrong type
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagCurrentNodeWrongType() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, "blah");
        try {
            nodeFormatterTag.doStartTag();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link HtmlAnchorNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>current style context attribute has wrong type
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagCurrentStyleWrongType() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, new Integer(6));
        try {
            nodeFormatterTag.doStartTag();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link HtmlAnchorNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>node exists, but no format is set
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTag() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, currentNode);
        assertEquals(TagSupport.SKIP_BODY, nodeFormatterTag.doStartTag());
    }

    /**
     * This method is called by the cactus framework after {@link #testDoStartTag()} has been executed to verify the
     * output generated by {@link #testDoStartTag()}. Thus it indirectly tests {@link
     * HtmlAnchorNodeFormatterTag#doStartTag()}
     * <p/>
     * <b>Special scenario:</b>node exists, but no format is set
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoStartTag(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals("<a href=\"" + URL + "\">" + TITLE + "</a>", resp.getText());
    }

    /**
     * This method tests the correct behavior of {@link HtmlAnchorNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>node exists, but has no title
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagNoTitle() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, new BreadCrumbNode(URL, null));
        assertEquals(TagSupport.SKIP_BODY, nodeFormatterTag.doStartTag());
    }

    /**
     * This method is called by the cactus framework after {@link #testDoStartTagNoTitle()} has been executed to verify
     * the output generated by {@link #testDoStartTag()}. Thus it indirectly tests {@link
     * HtmlAnchorNodeFormatterTag#doStartTag()}
     * <p/>
     * <b>Special scenario:</b>node exists, but has no title
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoStartTagNoTitle(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals("<a href=\"" + URL + "\">Unspecified</a>", resp.getText());
    }

    /**
     * This method tests the correct behavior of {@link HtmlAnchorNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>node and format are set
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagWithFormat() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, currentNode);
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, FORMAT);
        assertEquals(TagSupport.SKIP_BODY, nodeFormatterTag.doStartTag());
    }

    /**
     * This method is called by the cactus framework after {@link #testDoStartTagWithFormat()} has been executed to
     * verify the output generated by {@link #testDoStartTag()}. Thus it indirectly tests {@link
     * HtmlAnchorNodeFormatterTag#doStartTag()}
     * <p/>
     * <b>Special scenario:</b>node and format are set
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoStartTagWithFormat(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals("<a class=\"" + FORMAT + "\" href=\"" + URL + "\">" + TITLE + "</a>", resp.getText());
    }

    /**
     * This method tests the correct behavior of {@link HtmlAnchorNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>node and format are set, but format is empty string
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagWithEmptyFormat() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, currentNode);
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, "    ");
        assertEquals(TagSupport.SKIP_BODY, nodeFormatterTag.doStartTag());
    }

    /**
     * This method is called by the cactus framework after {@link #testDoStartTagWithEmptyFormat()} has been executed to
     * verify the output generated by {@link #testDoStartTag()}. Thus it indirectly tests {@link
     * HtmlAnchorNodeFormatterTag#doStartTag()}
     * <p/>
     * <b>Special scenario:</b>node and format are set, but format is empty string
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoStartTagWithEmptyFormat(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals("<a href=\"" + URL + "\">" + TITLE + "</a>", resp.getText());
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        nodeFormatterTag = new HtmlAnchorNodeFormatterTag();
        nodeFormatterTag.setPageContext(pageContext);
        pageContext.removeAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE);
        currentNode = new BreadCrumbNode(URL, TITLE);
    }
}
