/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers.RegexNodeMatcher;
import junit.framework.TestCase;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * This class contains all test cases for {@link DiscoveryUtils}.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class DiscoveryUtilsTest extends TestCase {
    /**
     * This is a site map used in test cases, created in {@link #setUp()}.
     */
    private Map siteMap;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher1;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher2;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher3;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher4;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher5;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher6;

    /**
     * This method tests the correct behavior of {@link DiscoveryUtils#createReverseMap(java.util.Map)}.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testCreateReverseMap() throws Exception {
        final Map reverseMap = DiscoveryUtils.createReverseMap(siteMap);
        assertEquals(4, reverseMap.size());
        assertEquals(null, reverseMap.get(nodeMatcher1));
        assertEquals(null, reverseMap.get(nodeMatcher5));

        List list = (List) reverseMap.get(nodeMatcher2);
        assertEquals(1, list.size());
        assertTrue(list.contains(nodeMatcher1));

        list = (List) reverseMap.get(nodeMatcher3);
        assertEquals(1, list.size());
        assertTrue(list.contains(nodeMatcher1));

        list = (List) reverseMap.get(nodeMatcher6);
        assertEquals(1, list.size());
        assertTrue(list.contains(nodeMatcher5));

        list = (List) reverseMap.get(nodeMatcher4);
        assertEquals(2, list.size());
        assertTrue(list.contains(nodeMatcher5));
        assertTrue(list.contains(nodeMatcher3));
    }

    /**
     * This method tests the correct failure behavior of {@link DiscoveryUtils#createReverseMap(java.util.Map)}.
     * <p/>
     * <b>Failure scenario:</b>given map is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testCreateReverseMapNullMap() throws Exception {
        try {
            DiscoveryUtils.createReverseMap(null);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DiscoveryUtils#createReverseMap(java.util.Map)}.
     * <p/>
     * <b>Failure scenario:</b>given map contains illegal key
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testCreateReverseMapIllegalKey() throws Exception {
        siteMap.put("a", new ArrayList());
        try {
            DiscoveryUtils.createReverseMap(siteMap);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DiscoveryUtils#createReverseMap(java.util.Map)}.
     * <p/>
     * <b>Failure scenario:</b>given map contains illegal value
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testCreateReverseMapIllegalValue() throws Exception {
        siteMap.put(nodeMatcher1, "a");
        try {
            DiscoveryUtils.createReverseMap(siteMap);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DiscoveryUtils#createReverseMap(java.util.Map)}.
     * <p/>
     * <b>Failure scenario:</b>given map contains illegal list element
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testCreateReverseMapIllegalListElement() throws Exception {
        siteMap.put(nodeMatcher1, Arrays.asList(new Object[]{"a"}));
        try {
            DiscoveryUtils.createReverseMap(siteMap);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link DiscoveryUtils#findShortestPaths(java.util.List,
     *java.util.Map)}.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testFindShortestPaths() throws Exception {
        final Map reverseMap = DiscoveryUtils.createReverseMap(siteMap);

        // find shortest path, 1, is a root node
        List shortestPaths = DiscoveryUtils.findShortestPaths(Arrays.asList(new Object[]{nodeMatcher1, nodeMatcher2}),
            reverseMap);
        assertEquals(1, shortestPaths.size());
        List firstPath = (List) shortestPaths.get(0);
        assertEquals(1, firstPath.size());
        assertTrue(firstPath.contains(nodeMatcher1));

        // 4 has path via 3,1 or directly to 5, so 5 is found shorter
        shortestPaths = DiscoveryUtils.findShortestPaths(Arrays.asList(new Object[]{nodeMatcher4}), reverseMap);
        assertEquals(1, shortestPaths.size());
        firstPath = (List) shortestPaths.get(0);
        assertEquals(2, firstPath.size());
        assertTrue(firstPath.contains(nodeMatcher4));
        assertTrue(firstPath.contains(nodeMatcher5));

        // 2 and 3 are both one away from 1
        shortestPaths = DiscoveryUtils.findShortestPaths(Arrays.asList(new Object[]{nodeMatcher2, nodeMatcher3}),
            reverseMap);
        assertEquals(2, shortestPaths.size());
        firstPath = (List) shortestPaths.get(0);
        final List secondPath = (List) shortestPaths.get(1);
        assertEquals(2, firstPath.size());
        assertEquals(2, secondPath.size());
        assertTrue(firstPath.contains(nodeMatcher1));
        assertTrue(secondPath.contains(nodeMatcher1));
        assertTrue(secondPath.contains(nodeMatcher2) || firstPath.contains(nodeMatcher2));
        assertTrue(secondPath.contains(nodeMatcher3) || firstPath.contains(nodeMatcher3));
    }

    /**
     * This method tests the correct failure behavior of {@link DiscoveryUtils#findShortestPaths(java.util.List,
     *java.util.Map)}.
     * <p/>
     * <b>Failure scenario:</b>given list is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testFindShortestPathNullList() throws Exception {
        try {
            DiscoveryUtils.findShortestPaths(null, DiscoveryUtils.createReverseMap(siteMap));
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DiscoveryUtils#findShortestPaths(java.util.List,
     *java.util.Map)}.
     * <p/>
     * <b>Failure scenario:</b>given list is empty
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testFindShortestPathEmptyList() throws Exception {
        try {
            DiscoveryUtils.findShortestPaths(new ArrayList(), DiscoveryUtils.createReverseMap(siteMap));
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DiscoveryUtils#findShortestPaths(java.util.List,
     *java.util.Map)}.
     * <p/>
     * <b>Failure scenario:</b>given list contains object that is not of type NodeMatcher
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testFindShortestPathWrongObjectInList() throws Exception {
        try {
            DiscoveryUtils.findShortestPaths(Arrays.asList(new Object[]{"a"}), DiscoveryUtils.createReverseMap(
                siteMap));
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DiscoveryUtils#findShortestPaths(java.util.List,
     *java.util.Map)}.
     * <p/>
     * <b>Failure scenario:</b>given map is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testFindShortestPathNullMap() throws Exception {
        try {
            DiscoveryUtils.findShortestPaths(Arrays.asList(new Object[]{nodeMatcher1}), null);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DiscoveryUtils#findShortestPaths(java.util.List,
     *java.util.Map)}.
     * <p/>
     * <b>Failure scenario:</b>given map is contains a wrong value
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testFindShortestPathWrongValueInMap() throws Exception {
        siteMap.put(nodeMatcher1, "aa");
        try {
            DiscoveryUtils.findShortestPaths(Arrays.asList(new Object[]{nodeMatcher1}), siteMap);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DiscoveryUtils#findShortestPaths(java.util.List,
     *java.util.Map)}.
     * <p/>
     * <b>Failure scenario:</b>given map is contains a wrong value in the list stored in the map
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testFindShortestPathWrongValueInMapsList() throws Exception {
        siteMap.put(nodeMatcher1, Arrays.asList(new Object[]{"aa"}));
        try {
            DiscoveryUtils.findShortestPaths(Arrays.asList(new Object[]{nodeMatcher1}), siteMap);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link DiscoveryUtils#getFirstPath(java.util.List)}.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetFirstPath() throws Exception {
        final Map reverseMap = DiscoveryUtils.createReverseMap(siteMap);
        // finds 1-2 as 2's title is lexicographically less than 3's title
        List shortestPaths = DiscoveryUtils.findShortestPaths(Arrays.asList(new Object[]{nodeMatcher2, nodeMatcher3}),
            reverseMap);
        List firstPath = DiscoveryUtils.getFirstPath(shortestPaths);
        assertEquals(2, firstPath.size());
        assertTrue(firstPath.contains(nodeMatcher1));
        assertTrue(firstPath.contains(nodeMatcher2));

        // finds 5-6 as 6's title[null] is lexicographically less than 4's title
        shortestPaths = DiscoveryUtils.findShortestPaths(Arrays.asList(new Object[]{nodeMatcher4, nodeMatcher6}),
            reverseMap);
        firstPath = DiscoveryUtils.getFirstPath(shortestPaths);
        assertEquals(2, firstPath.size());
        assertTrue(firstPath.contains(nodeMatcher6));
        assertTrue(firstPath.contains(nodeMatcher5));
    }

    /**
     * This method tests the correct failure behavior of {@link DiscoveryUtils#getFirstPath(java.util.List)}.
     * <p/>
     * <b>Failure scenario:</b>given list is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetFirstPathNullList() throws Exception {
        try {
            DiscoveryUtils.getFirstPath(null);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DiscoveryUtils#getFirstPath(java.util.List)}.
     * <p/>
     * <b>Failure scenario:</b>given list is empty
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetFirstPathEmptyList() throws Exception {
        try {
            DiscoveryUtils.getFirstPath(new ArrayList());
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DiscoveryUtils#getFirstPath(java.util.List)}.
     * <p/>
     * <b>Failure scenario:</b>given list contains empty lists
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetFirstPathListContainsEmptyLists() throws Exception {
        try {
            DiscoveryUtils.getFirstPath(Arrays.asList(new Object[]{
                new ArrayList(),
                new ArrayList(),
            }));
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DiscoveryUtils#getFirstPath(java.util.List)}.
     * <p/>
     * <b>Failure scenario:</b>given list contains lists of different size
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetFirstPathListContainsListsOfDifferentSize() throws Exception {
        try {
            DiscoveryUtils.getFirstPath(Arrays.asList(new Object[]{
                Arrays.asList(new Object[]{nodeMatcher1}),
                Arrays.asList(new Object[]{nodeMatcher1, nodeMatcher2}),
            }));
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DiscoveryUtils#getFirstPath(java.util.List)}.
     * <p/>
     * <b>Failure scenario:</b>given list contains lists that contain wrong elements
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetFirstPathListContainsListsWithWrongElements() throws Exception {
        try {
            DiscoveryUtils.getFirstPath(Arrays.asList(new Object[]{
                Arrays.asList(new Object[]{"a"}),
                Arrays.asList(new Object[]{nodeMatcher1}),
            }));
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();

        nodeMatcher1 = new RegexNodeMatcher("aa1", "uu1", "uu1");
        nodeMatcher2 = new RegexNodeMatcher("aa2", "uu2", "uu2");
        nodeMatcher3 = new RegexNodeMatcher("aa3", "uu3", "uu3");
        nodeMatcher4 = new RegexNodeMatcher("aa4", "uu4", "uu4");
        nodeMatcher5 = new RegexNodeMatcher("aa5", "uu5", "uu5");
        nodeMatcher6 = new RegexNodeMatcher("a", "uu6", "uu6");

        siteMap = new HashMap();
        siteMap.put(nodeMatcher1, Arrays.asList(new Object[]{nodeMatcher2, nodeMatcher3}));
        siteMap.put(nodeMatcher3, Arrays.asList(new Object[]{nodeMatcher4}));
        siteMap.put(nodeMatcher5, Arrays.asList(new Object[]{nodeMatcher6, nodeMatcher4}));
    }
}
