/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BaseBreadCrumbTrailTagTest;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers.RegexNodeMatcher;

import java.util.List;
import java.util.Map;


/**
 * This class contains all test cases for {@link XmlPathDiscovery}.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class XmlPathDiscoveryTest extends BaseBreadCrumbTrailTagTest {
    /**
     * This is the path to a valid bread crumbs sitemap file used in test cases.
     */
    private static final String FILENAME = "testcaseBreadCrumbs.xml";
    /**
     * This is the path to an invalid bread crumbs sitemap file used in test cases.
     */
    private static final String INVALID_FILENAME = "invalid_testcaseBreadCrumbs.xml";
    /**
     * This is the path to a malformed bread crumbs sitemap file used in test cases.
     */
    private static final String MALFORMED_FILENAME = "malformed_testcaseBreadCrumbs.xml";
    /**
     * This is the path to an invalid bread crumbs sitemap file used in test cases.
     */
    private static final String DUPLICATE_CHILDLIST_FILENAME = "duplicateChildList_testcaseBreadCrumbs.xml";

    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher1;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher2;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher3;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher4;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher5;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher6;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher7;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher8;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher9;

    /**
     * This method tests the correct behavior of {@link XmlPathDiscovery#XmlPathDiscovery(String)}.
     */
    public void testOneArgCtor() {
        new XmlPathDiscovery(FILENAME);
    }

    /**
     * This method tests the correct failure behavior of {@link XmlPathDiscovery#XmlPathDiscovery(String)}.
     * <p/>
     * <b>Failure scenario:</b>given filename is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testOneArgCtorFailNull() throws Exception {
        try {
            new XmlPathDiscovery(null);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlPathDiscovery#XmlPathDiscovery(String)}.
     * <p/>
     * <b>Failure scenario:</b>given filename is an empty String
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testOneArgCtorFailEmpty() throws Exception {
        try {
            new XmlPathDiscovery(" ");
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link XmlPathDiscovery#XmlPathDiscovery(String,boolean)}.
     */
    public void testTwoArgCtor() {
        new XmlPathDiscovery(FILENAME, true);
    }

    /**
     * This method tests the correct failure behavior of {@link XmlPathDiscovery#XmlPathDiscovery(String,boolean)}.
     * <p/>
     * <b>Failure scenario:</b>given filename is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testTwoArgCtorFailNull() throws Exception {
        try {
            new XmlPathDiscovery(null, true);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlPathDiscovery#XmlPathDiscovery(String,boolean)}.
     * <p/>
     * <b>Failure scenario:</b>given filename is an empty String
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testTwoArgCtorFailEmpty() throws Exception {
        try {
            new XmlPathDiscovery(" ", true);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link XmlPathDiscovery#getSiteMap()}.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetSiteMap() throws Exception {
        final XmlPathDiscovery xmlPathDiscovery = new XmlPathDiscovery(FILENAME);
        final Map siteMap = xmlPathDiscovery.getSiteMap();
        assertEquals(9, siteMap.size());
        assertNull(siteMap.get(nodeMatcher2));
        assertNull(siteMap.get(nodeMatcher4));

        List list = (List) siteMap.get(nodeMatcher1);
        assertEquals(2, list.size());
        assertTrue(list.contains(nodeMatcher2));
        assertTrue(list.contains(nodeMatcher3));

        list = (List) siteMap.get(nodeMatcher3);
        assertEquals(1, list.size());
        assertTrue(list.contains(nodeMatcher4));

        list = (List) siteMap.get(nodeMatcher5);
        assertEquals(2, list.size());
        assertTrue(list.contains(nodeMatcher4));
        assertTrue(list.contains(nodeMatcher6));

        list = (List) siteMap.get(nodeMatcher6);
        assertEquals(1, list.size());
        assertTrue(list.contains(nodeMatcher7));

        list = (List) siteMap.get(nodeMatcher7);
        assertEquals(1, list.size());
        assertTrue(list.contains(nodeMatcher6));

        list = (List) siteMap.get(nodeMatcher8);
        assertEquals(1, list.size());
        assertTrue(list.contains(nodeMatcher9));

        list = (List) siteMap.get(nodeMatcher9);
        assertEquals(1, list.size());
        assertTrue(list.contains(nodeMatcher8));
    }

    /**
     * This method tests the correct failure behavior of {@link XmlPathDiscovery#getSiteMap()}.
     * <p/>
     * <b>Failure scenario:</b>given file contains invalid XML (missing attributes)
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetSiteMapFailureInvalidXML() throws Exception {
        try {
            new XmlPathDiscovery(INVALID_FILENAME).getSiteMap();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlPathDiscovery#getSiteMap()}.
     * <p/>
     * <b>Failure scenario:</b>given file contains malformed XML
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetSiteMapFailureMalformedXML() throws Exception {
        try {
            new XmlPathDiscovery(MALFORMED_FILENAME).getSiteMap();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlPathDiscovery#getSiteMap()}.
     * <p/>
     * <b>Failure scenario:</b>given file contains duplicate child list for an identical node
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetSiteMapFailureDuplicateChildListXML() throws Exception {
        try {
            new XmlPathDiscovery(DUPLICATE_CHILDLIST_FILENAME).getSiteMap();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link XmlPathDiscovery#getSiteMap()} when reading in the data from a
     * string instead of a file.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetSiteMapFromString() throws Exception {
        final XmlPathDiscovery xmlPathDiscovery = new XmlPathDiscovery(readResourceIntoString(FILENAME), false);
        final Map siteMap = xmlPathDiscovery.getSiteMap();
        assertEquals(9, siteMap.size());
        assertNull(siteMap.get(nodeMatcher2));
        assertNull(siteMap.get(nodeMatcher4));

        List list = (List) siteMap.get(nodeMatcher1);
        assertEquals(2, list.size());
        assertTrue(list.contains(nodeMatcher2));
        assertTrue(list.contains(nodeMatcher3));

        list = (List) siteMap.get(nodeMatcher3);
        assertEquals(1, list.size());
        assertTrue(list.contains(nodeMatcher4));

        list = (List) siteMap.get(nodeMatcher5);
        assertEquals(2, list.size());
        assertTrue(list.contains(nodeMatcher4));
        assertTrue(list.contains(nodeMatcher6));

        list = (List) siteMap.get(nodeMatcher6);
        assertEquals(1, list.size());
        assertTrue(list.contains(nodeMatcher7));

        list = (List) siteMap.get(nodeMatcher7);
        assertEquals(1, list.size());
        assertTrue(list.contains(nodeMatcher6));

        list = (List) siteMap.get(nodeMatcher8);
        assertEquals(1, list.size());
        assertTrue(list.contains(nodeMatcher9));

        list = (List) siteMap.get(nodeMatcher9);
        assertEquals(1, list.size());
        assertTrue(list.contains(nodeMatcher8));
    }

    /**
     * This method tests the correct failure behavior of {@link XmlPathDiscovery#getSiteMap()}  when reading in the data
     * from a string instead of a file.
     * <p/>
     * <b>Failure scenario:</b>given file contains invalid XML (missing attributes)
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetSiteMapFromStringFailureInvalidXML() throws Exception {
        try {
            new XmlPathDiscovery(readResourceIntoString(INVALID_FILENAME), false).getSiteMap();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlPathDiscovery#getSiteMap()} when reading in the data
     * from a string instead of a file.
     * <p/>
     * <b>Failure scenario:</b>given file contains malformed XML
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetSiteMapFromStringFailureMalformedXML() throws Exception {
        try {
            new XmlPathDiscovery(readResourceIntoString(MALFORMED_FILENAME), false).getSiteMap();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlPathDiscovery#getSiteMap()} when reading in the data
     * from a string instead of a file.
     * <p/>
     * <b>Failure scenario:</b>given file contains duplicate child list for an identical node
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetSiteMapFromStringFailureDuplicateChildlistXML() throws Exception {
        try {
            new XmlPathDiscovery(readResourceIntoString(DUPLICATE_CHILDLIST_FILENAME), false).getSiteMap();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link XmlPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)}.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPath() throws Exception {
        final XmlPathDiscovery xmlPathDiscovery = new XmlPathDiscovery(FILENAME);

        // test root
        BreadCrumbNode[] path = xmlPathDiscovery.getPath(new BreadCrumbNode(nodeMatcher1.getUrl(), null), pageContext);
        assertEquals(1, path.length);
        assertEquals(nodeMatcher1.getTitle(), path[0].getTitle());
        assertEquals(nodeMatcher1.getUrl(), path[0].getUrl());

        // test not found
        path = xmlPathDiscovery.getPath(new BreadCrumbNode("blah", null), pageContext);
        assertEquals(null, path);

        // test normal
        path = xmlPathDiscovery.getPath(new BreadCrumbNode(nodeMatcher2.getUrl(), null), pageContext);
        assertEquals(2, path.length);
        assertEquals(nodeMatcher1.getTitle(), path[0].getTitle());
        assertEquals(nodeMatcher1.getUrl(), path[0].getUrl());
        assertEquals(nodeMatcher2.getTitle(), path[1].getTitle());
        assertEquals(nodeMatcher2.getUrl(), path[1].getUrl());

        // test shortest path
        path = xmlPathDiscovery.getPath(new BreadCrumbNode(nodeMatcher4.getUrl(), null), pageContext);
        assertEquals(2, path.length);
        assertEquals(nodeMatcher5.getTitle(), path[0].getTitle());
        assertEquals(nodeMatcher5.getUrl(), path[0].getUrl());
        assertEquals(nodeMatcher4.getTitle(), path[1].getTitle());
        assertEquals(nodeMatcher4.getUrl(), path[1].getUrl());

        // test cycle in path
        path = xmlPathDiscovery.getPath(new BreadCrumbNode(nodeMatcher7.getUrl(), null), pageContext);
        assertEquals(3, path.length);
        assertEquals(nodeMatcher5.getTitle(), path[0].getTitle());
        assertEquals(nodeMatcher5.getUrl(), path[0].getUrl());
        assertEquals(nodeMatcher6.getTitle(), path[1].getTitle());
        assertEquals(nodeMatcher6.getUrl(), path[1].getUrl());
        assertEquals(nodeMatcher7.getTitle(), path[2].getTitle());
        assertEquals(nodeMatcher7.getUrl(), path[2].getUrl());

        // test two nodes build a cycle and have no root
        path = xmlPathDiscovery.getPath(new BreadCrumbNode(nodeMatcher8.getUrl(), null), pageContext);
        assertEquals(null, path);
    }

    /**
     * This method tests the correct failure behavior of {@link XmlPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)} .
     * <p/>
     * <b>Failure scenario:</b>currentNode arg is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPathFailureNullNode() throws Exception {
        try {
            new XmlPathDiscovery(FILENAME).getPath(null, pageContext);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)} .
     * <p/>
     * <b>Failure scenario:</b>pageContext arg is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPathFailureNullPageContext() throws Exception {
        try {
            new XmlPathDiscovery(FILENAME).getPath(new BreadCrumbNode("aa", null), null);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        nodeMatcher1 = new RegexNodeMatcher("aa1", "uu1", "uu1");
        nodeMatcher2 = new RegexNodeMatcher("aa2", "uu2", "uu2");
        nodeMatcher3 = new RegexNodeMatcher("aa3", "uu3", "uu3");
        nodeMatcher4 = new RegexNodeMatcher("aa4", "uu4", "uu4");
        nodeMatcher5 = new RegexNodeMatcher("aa5", "uu5", "uu5");
        nodeMatcher6 = new RegexNodeMatcher("a", "uu6", "uu6");
        nodeMatcher7 = new RegexNodeMatcher("aa7", "uu7", "uu7");
        nodeMatcher8 = new RegexNodeMatcher("aa8", "uu8", "uu8");
        nodeMatcher9 = new RegexNodeMatcher("aa9", "uu9", "uu9");

    }
}
